;;; ic-read-multiple.el --- Alternative to completing-read-multiple -*- lexical-binding: t; -*-

;;; Commentary:

;; <return> - add a candidate to the list of chosen
;; C-<backspace> - delete the last chosen candidate
;; C-<return> - exit completion, return the list of chosen

;; (ic-completing-read-multiple
;;  "PROMPT: "
;;  (list "first" "sec,ond" "third" "forth" "fifth"))

;;; Code:

(defun ic-completing-erase-chosen ()
  (interactive)
  (throw 'ic-completion 'erase))

(defun ic-completing-end-completion ()
  (interactive)
  (throw 'ic-completion 'end))

(define-minor-mode ic-completing-multiple-minor-mode
  "Toggle ic-completing-read minor mode. "
  :keymap
  (define-keymap
    "C-<backspace>" #'ic-completing-erase-chosen
    "C-<return>" #'ic-completing-end-completion))

(defun ic-completing-read
    (prompt collection &optional list-of-chosen
	    predicate require-match initial-input
	    hist def inherit-input-method)
  (declare (indent 2))
  (minibuffer-with-setup-hook
      #'ic-completing-multiple-minor-mode
    (catch 'ic-completion
      (completing-read
       (concat prompt
	       (when list-of-chosen
		 (concat
		  "[" (string-join
		       (reverse list-of-chosen)
		       (propertize "," 'face 'error))
		  "] ")))
       collection predicate require-match initial-input
       hist def inherit-input-method))))

;;;###autoload
(defun ic-completing-read-multiple
    (prompt collection &optional predicate require-match initial-input
	    hist def inherit-input-method)
  (let (list-of-chosen done)
    (while (not done)
      (pcase (ic-completing-read prompt collection
	       list-of-chosen predicate require-match
	       initial-input hist def inherit-input-method)
	(`erase
	 (when list-of-chosen
	   (push (pop list-of-chosen) collection)))
	(`end (setq done t))
	((and chosen _)
	 (push chosen list-of-chosen)
	 (setq collection (remove chosen collection)))))
    (or list-of-chosen def)))

;; CHECK
;; (ic-completing-read-multiple "Prompt: " (list "first" "second" "third"))

(provide 'ic-read-multiple)

;;; ic-read-multiple.el ends here
